@@ -26,9 +26,10 @@
 #include <string.h>
 #include <unistd.h>
 
-#include "common.h"
 #include "compat.h"
 #include "log.h"
+#include "ly_common.h"
+#include "metadata.h"
 #include "printer_data.h"
 #include "tree_data.h"
 #include "tree_schema.h"
@@ -54,6 +55,20 @@ lyd_node_should_print(const struct lyd_node *node, uint32_t options)
                 /* explicit default node */
                 return 0;
             }
+        } else if (lysc_is_np_cont(node->schema)) {
+            if (options & LYD_PRINT_KEEPEMPTYCONT) {
+                /* explicit request to print, redundant to check */
+                return 1;
+            }
+
+            LY_LIST_FOR(lyd_child(node), elem) {
+                if (lyd_node_should_print(elem, options)) {
+                    return 1;
+                }
+            }
+
+            /* NP container without any printed children (such as other NP containers with only nodes set to their default values) */
+            return 0;
         }
     } else if ((node->flags & LYD_DEFAULT) && (node->schema->nodetype == LYS_CONTAINER)) {
         if (options & LYD_PRINT_KEEPEMPTYCONT) {
@@ -89,6 +104,12 @@ lyd_node_should_print(const struct lyd_node *node, uint32_t options)
     return 1;
 }
 
+LIBYANG_API_DEF ly_bool
+lyd_metadata_should_print(const struct lyd_meta *meta)
+{
+    return !lyd_meta_is_internal(meta);
+}
+
 LIBYANG_API_DEF LY_OUT_TYPE
 ly_out_type(const struct ly_out *out)
 {
@@ -437,6 +458,7 @@ ly_vprint_(struct ly_out *out, const char *format, va_list ap)
             if (!*out->method.mem.buf) {
                 out->method.mem.len = 0;
                 out->method.mem.size = 0;
+                free(msg);
                 LOGMEM(NULL);
                 return LY_EMEM;
             }
