@@ -23,8 +23,8 @@
 #include "libyang.h"
 
 /* additional internal headers for some useful simple macros */
-#include "common.h"
 #include "compat.h"
+#include "ly_common.h"
 #include "plugins_internal.h" /* LY_TYPE_*_STR */
 
 /**
@@ -50,11 +50,11 @@ lyplg_type_store_int(const struct ly_ctx *ctx, const struct lysc_type *type, con
         const struct lysc_node *UNUSED(ctx_node), struct lyd_value *storage, struct lys_glob_unres *UNUSED(unres),
         struct ly_err_item **err)
 {
+    struct lysc_type_num *type_num = (struct lysc_type_num *)type;
     LY_ERR ret = LY_SUCCESS;
     int64_t num = 0;
     int base = 1;
     char *canon = NULL;
-    struct lysc_type_num *type_num = (struct lysc_type_num *)type;
 
     /* init storage */
     memset(storage, 0, sizeof *storage);
@@ -154,11 +154,13 @@ lyplg_type_store_int(const struct ly_ctx *ctx, const struct lysc_type *type, con
         LY_CHECK_GOTO(ret, cleanup);
     }
 
-    /* validate range of the number */
-    if (type_num->range) {
-        ret = lyplg_type_validate_range(type->basetype, type_num->range, num, storage->_canonical,
-                strlen(storage->_canonical), err);
-        LY_CHECK_GOTO(ret, cleanup);
+    if (!(options & LYPLG_TYPE_STORE_ONLY)) {
+        /* validate range of the number */
+        if (type_num->range) {
+            ret = lyplg_type_validate_range(type->basetype, type_num->range, num, storage->_canonical,
+                    strlen(storage->_canonical), err);
+            LY_CHECK_GOTO(ret, cleanup);
+        }
     }
 
 cleanup:
@@ -173,7 +175,7 @@ cleanup:
 }
 
 LIBYANG_API_DEF LY_ERR
-lyplg_type_compare_int(const struct lyd_value *val1, const struct lyd_value *val2)
+lyplg_type_compare_int(const struct ly_ctx *UNUSED(ctx), const struct lyd_value *val1, const struct lyd_value *val2)
 {
     if (val1->realtype != val2->realtype) {
         return LY_ENOT;
@@ -206,6 +208,52 @@ lyplg_type_compare_int(const struct lyd_value *val1, const struct lyd_value *val
     return LY_SUCCESS;
 }
 
+LIBYANG_API_DEF int
+lyplg_type_sort_int(const struct ly_ctx *UNUSED(ctx), const struct lyd_value *val1, const struct lyd_value *val2)
+{
+    switch (val1->realtype->basetype) {
+    case LY_TYPE_INT8:
+        if (val1->int8 < val2->int8) {
+            return -1;
+        } else if (val1->int8 > val2->int8) {
+            return 1;
+        } else {
+            return 0;
+        }
+        break;
+    case LY_TYPE_INT16:
+        if (val1->int16 < val2->int16) {
+            return -1;
+        } else if (val1->int16 > val2->int16) {
+            return 1;
+        } else {
+            return 0;
+        }
+        break;
+    case LY_TYPE_INT32:
+        if (val1->int32 < val2->int32) {
+            return -1;
+        } else if (val1->int32 > val2->int32) {
+            return 1;
+        } else {
+            return 0;
+        }
+        break;
+    case LY_TYPE_INT64:
+        if (val1->int64 < val2->int64) {
+            return -1;
+        } else if (val1->int64 > val2->int64) {
+            return 1;
+        } else {
+            return 0;
+        }
+        break;
+    default:
+        break;
+    }
+    return 0;
+}
+
 LIBYANG_API_DEF const void *
 lyplg_type_print_int(const struct ly_ctx *UNUSED(ctx), const struct lyd_value *value, LY_VALUE_FORMAT format,
         void *UNUSED(prefix_data), ly_bool *dynamic, size_t *value_len)
@@ -268,11 +316,11 @@ lyplg_type_store_uint(const struct ly_ctx *ctx, const struct lysc_type *type, co
         const struct lysc_node *UNUSED(ctx_node), struct lyd_value *storage, struct lys_glob_unres *UNUSED(unres),
         struct ly_err_item **err)
 {
+    struct lysc_type_num *type_num = (struct lysc_type_num *)type;
     LY_ERR ret = LY_SUCCESS;
     uint64_t num = 0;
     int base = 0;
     char *canon;
-    struct lysc_type_num *type_num = (struct lysc_type_num *)type;
 
     /* init storage */
     memset(storage, 0, sizeof *storage);
@@ -352,11 +400,13 @@ lyplg_type_store_uint(const struct ly_ctx *ctx, const struct lysc_type *type, co
         LY_CHECK_GOTO(ret, cleanup);
     }
 
-    /* validate range of the number */
-    if (type_num->range) {
-        ret = lyplg_type_validate_range(type->basetype, type_num->range, num, storage->_canonical,
-                strlen(storage->_canonical), err);
-        LY_CHECK_GOTO(ret, cleanup);
+    if (!(options & LYPLG_TYPE_STORE_ONLY)) {
+        /* validate range of the number */
+        if (type_num->range) {
+            ret = lyplg_type_validate_range(type->basetype, type_num->range, num, storage->_canonical,
+                    strlen(storage->_canonical), err);
+            LY_CHECK_GOTO(ret, cleanup);
+        }
     }
 
 cleanup:
@@ -371,7 +421,7 @@ cleanup:
 }
 
 LIBYANG_API_DEF LY_ERR
-lyplg_type_compare_uint(const struct lyd_value *val1, const struct lyd_value *val2)
+lyplg_type_compare_uint(const struct ly_ctx *UNUSED(ctx), const struct lyd_value *val1, const struct lyd_value *val2)
 {
     switch (val1->realtype->basetype) {
     case LY_TYPE_UINT8:
@@ -400,6 +450,52 @@ lyplg_type_compare_uint(const struct lyd_value *val1, const struct lyd_value *va
     return LY_SUCCESS;
 }
 
+LIBYANG_API_DEF int
+lyplg_type_sort_uint(const struct ly_ctx *UNUSED(ctx), const struct lyd_value *val1, const struct lyd_value *val2)
+{
+    switch (val1->realtype->basetype) {
+    case LY_TYPE_UINT8:
+        if (val1->uint8 < val2->uint8) {
+            return -1;
+        } else if (val1->uint8 > val2->uint8) {
+            return 1;
+        } else {
+            return 0;
+        }
+        break;
+    case LY_TYPE_UINT16:
+        if (val1->uint16 < val2->uint16) {
+            return -1;
+        } else if (val1->uint16 > val2->uint16) {
+            return 1;
+        } else {
+            return 0;
+        }
+        break;
+    case LY_TYPE_UINT32:
+        if (val1->uint32 < val2->uint32) {
+            return -1;
+        } else if (val1->uint32 > val2->uint32) {
+            return 1;
+        } else {
+            return 0;
+        }
+        break;
+    case LY_TYPE_UINT64:
+        if (val1->uint64 < val2->uint64) {
+            return -1;
+        } else if (val1->uint64 > val2->uint64) {
+            return 1;
+        } else {
+            return 0;
+        }
+        break;
+    default:
+        break;
+    }
+    return 0;
+}
+
 LIBYANG_API_DEF const void *
 lyplg_type_print_uint(const struct ly_ctx *UNUSED(ctx), const struct lyd_value *value, LY_VALUE_FORMAT format,
         void *UNUSED(prefix_data), ly_bool *dynamic, size_t *value_len)
@@ -473,7 +569,7 @@ const struct lyplg_type_record plugins_integer[] = {
         .plugin.store = lyplg_type_store_uint,
         .plugin.validate = NULL,
         .plugin.compare = lyplg_type_compare_uint,
-        .plugin.sort = NULL,
+        .plugin.sort = lyplg_type_sort_uint,
         .plugin.print = lyplg_type_print_uint,
         .plugin.duplicate = lyplg_type_dup_simple,
         .plugin.free = lyplg_type_free_simple,
@@ -487,7 +583,7 @@ const struct lyplg_type_record plugins_integer[] = {
         .plugin.store = lyplg_type_store_uint,
         .plugin.validate = NULL,
         .plugin.compare = lyplg_type_compare_uint,
-        .plugin.sort = NULL,
+        .plugin.sort = lyplg_type_sort_uint,
         .plugin.print = lyplg_type_print_uint,
         .plugin.duplicate = lyplg_type_dup_simple,
         .plugin.free = lyplg_type_free_simple,
@@ -501,7 +597,7 @@ const struct lyplg_type_record plugins_integer[] = {
         .plugin.store = lyplg_type_store_uint,
         .plugin.validate = NULL,
         .plugin.compare = lyplg_type_compare_uint,
-        .plugin.sort = NULL,
+        .plugin.sort = lyplg_type_sort_uint,
         .plugin.print = lyplg_type_print_uint,
         .plugin.duplicate = lyplg_type_dup_simple,
         .plugin.free = lyplg_type_free_simple,
@@ -515,7 +611,7 @@ const struct lyplg_type_record plugins_integer[] = {
         .plugin.store = lyplg_type_store_uint,
         .plugin.validate = NULL,
         .plugin.compare = lyplg_type_compare_uint,
-        .plugin.sort = NULL,
+        .plugin.sort = lyplg_type_sort_uint,
         .plugin.print = lyplg_type_print_uint,
         .plugin.duplicate = lyplg_type_dup_simple,
         .plugin.free = lyplg_type_free_simple,
@@ -529,7 +625,7 @@ const struct lyplg_type_record plugins_integer[] = {
         .plugin.store = lyplg_type_store_int,
         .plugin.validate = NULL,
         .plugin.compare = lyplg_type_compare_int,
-        .plugin.sort = NULL,
+        .plugin.sort = lyplg_type_sort_int,
         .plugin.print = lyplg_type_print_int,
         .plugin.duplicate = lyplg_type_dup_simple,
         .plugin.free = lyplg_type_free_simple,
@@ -543,7 +639,7 @@ const struct lyplg_type_record plugins_integer[] = {
         .plugin.store = lyplg_type_store_int,
         .plugin.validate = NULL,
         .plugin.compare = lyplg_type_compare_int,
-        .plugin.sort = NULL,
+        .plugin.sort = lyplg_type_sort_int,
         .plugin.print = lyplg_type_print_int,
         .plugin.duplicate = lyplg_type_dup_simple,
         .plugin.free = lyplg_type_free_simple,
@@ -557,7 +653,7 @@ const struct lyplg_type_record plugins_integer[] = {
         .plugin.store = lyplg_type_store_int,
         .plugin.validate = NULL,
         .plugin.compare = lyplg_type_compare_int,
-        .plugin.sort = NULL,
+        .plugin.sort = lyplg_type_sort_int,
         .plugin.print = lyplg_type_print_int,
         .plugin.duplicate = lyplg_type_dup_simple,
         .plugin.free = lyplg_type_free_simple,
@@ -571,7 +667,7 @@ const struct lyplg_type_record plugins_integer[] = {
         .plugin.store = lyplg_type_store_int,
         .plugin.validate = NULL,
         .plugin.compare = lyplg_type_compare_int,
-        .plugin.sort = NULL,
+        .plugin.sort = lyplg_type_sort_int,
         .plugin.print = lyplg_type_print_int,
         .plugin.duplicate = lyplg_type_dup_simple,
         .plugin.free = lyplg_type_free_simple,
