@@ -937,6 +937,7 @@ lys_compile_unres_dflt(struct lysc_ctx *ctx, struct lysc_node *node, struct lysc
 {
     LY_ERR ret;
     uint32_t options;
+    struct lyd_value *val;
     struct ly_err_item *err = NULL;
 
     options = (ctx->ctx->flags & LY_CTX_REF_IMPLEMENTED) ? LYPLG_TYPE_STORE_IMPLEMENT : 0;
@@ -963,11 +964,16 @@ lys_compile_unres_dflt(struct lysc_ctx *ctx, struct lysc_node *node, struct lysc
     }
 
     LY_ATOMIC_INC_BARRIER(((struct lysc_type *)storage->realtype)->refcount);
-    if (storage->realtype->basetype == LY_TYPE_INST) {
+    if (storage->realtype->basetype == LY_TYPE_UNION) {
+        val = &storage->subvalue->value;
+    } else {
+        val = storage;
+    }
+    if (val->realtype->basetype == LY_TYPE_INST) {
         /* ly_path includes references to other nodes, in case they are in foreign modules, the context would
          * need to be freed in specific order to avoid accessing freed memory, so just avoid storing it */
-        ly_path_free(storage->target);
-        storage->target = NULL;
+        ly_path_free(val->target);
+        val->target = NULL;
     }
     return LY_SUCCESS;
 }
@@ -1537,11 +1543,13 @@ lys_compile_depset_r(struct ly_ctx *ctx, struct ly_set *dep_set, struct lys_glob
         LY_CHECK_GOTO(ret = lys_compile(mod, &unres->ds_unres), cleanup);
     }
 
+resolve_unres:
     /* resolve dep set unres */
     ret = lys_compile_unres_depset(ctx, unres);
+    lys_compile_unres_depset_erase(ctx, unres);
+
     if (ret == LY_ERECOMPILE) {
-        /* new module is implemented, discard current dep set unres and recompile the whole dep set */
-        lys_compile_unres_depset_erase(ctx, unres);
+        /* new module is implemented referencing previously compiled modules, recompile the whole dep set */
         return lys_compile_depset_r(ctx, dep_set, unres);
     } else if (ret) {
         /* error */
@@ -1551,6 +1559,13 @@ lys_compile_depset_r(struct ly_ctx *ctx, struct ly_set *dep_set, struct lys_glob
     /* success, unset the flags of all the modules in the dep set */
     for (i = 0; i < dep_set->count; ++i) {
         mod = dep_set->objs[i];
+
+        if (mod->to_compile && !mod->compiled) {
+            /* new module is implemented but does not require recompilation of the whole dep set */
+            LY_CHECK_GOTO(ret = lys_compile(mod, &unres->ds_unres), cleanup);
+            goto resolve_unres;
+        }
+
         mod->to_compile = 0;
     }
 
