@@ -516,6 +516,7 @@ lyd_value_store(const struct ly_ctx *ctx, struct lyd_value *val, const struct ly
 
     if (!value) {
         value = "";
+        value_len = 0;
     }
     if (incomplete) {
         *incomplete = 0;
@@ -1626,9 +1627,6 @@ ly_time_tz_offset_at(time_t time)
     struct tm tm_local, tm_utc;
     int result = 0;
 
-    /* init timezone */
-    tzset();
-
     /* get local and UTC time */
     localtime_r(&time, &tm_local);
     gmtime_r(&time, &tm_utc);
@@ -1670,7 +1668,7 @@ ly_time_str2time(const char *value, time_t *time, char **fractions_s)
     int64_t shift, shift_m;
     time_t t;
 
-    LY_CHECK_ARG_RET(NULL, value, strlen(value) > 17, time, LY_EINVAL);
+    LY_CHECK_ARG_RET(NULL, value, strnlen(value, 18) > 17, time, LY_EINVAL);
 
     tm.tm_year = atoi(&value[0]) - 1900;
     tm.tm_mon = atoi(&value[5]) - 1;
@@ -1776,9 +1774,6 @@ ly_time_time2str(time_t time, const char *fractions_s, char **str)
 
     LY_CHECK_ARG_RET(NULL, str, LY_EINVAL);
 
-    /* init timezone */
-    tzset();
-
     /* convert */
     if (!localtime_r(&time, &tm)) {
         return LY_ESYS;
